# Before `make install' is performed this script should be runnable with
# `make test'. After `make install' it should work as `perl test.pl'

######################### We start with some black magic to print on failure.

# Change 1..1 below to 1..last_test_to_print .
# (It may become useful if the test is moved to ./t subdirectory.)

BEGIN { $| = 1; print "1..18\n"; }
END {print "not ok 1\n" unless $loaded;}
use Tie::PerlFiles;
$loaded = 1;
print "ok 1\n";

######################### End of black magic.

# Insert your test code below (better if it prints "ok 13"
# (correspondingly "not ok 13") depending on the success of chunk 13
# of the test code):

sub report_result {
        my $ok = shift;
        $TEST_NUM ||= 2;
        print "not " unless $ok;
        print "ok $TEST_NUM\n";
        print "@_\n" if (not $ok and $ENV{TEST_VERBOSE});
        $TEST_NUM++;
}

my $dir = "data";
my $ref = [ 1, 2, 3];
bless $ref, 'Goober';

# 2 = tie on a new db
&report_result( tie( %hash, 'Tie::PerlFiles', $dir, 'rw' ), $! );

# 3 = store a value, see if it exists
$hash{'test'} = $ref;
&report_result( (-e "$dir/test"), 'bad store on test key' );

# 4,5 = store a value with a weird key name
$hash{'../badkey'} = $ref; # bad ju-ju if this fails
&report_result( (! -e "$dir/../badkey"), 'stored weird key in wrong place' );
&report_result( (-e "$dir/%2e%2e%2fbadkey"), 'failed to store weird key' );

untie %hash;

# 6 = re-tie db
&report_result( tie( %hash, 'Tie::PerlFiles', $dir, 'rw' ), $! );

# 7,8 = check stored data
$val = $hash{'test'};
&report_result( ref($val) eq "Goober" , "lost our bless on stored value" );
&report_result( $$val[1] == 2 , "stored array didn't survive" );

# 9,10 = check exists
&report_result( exists $hash{'../badkey'}, 'weird key does not EXIST' );
&report_result( !exists $hash{''}, 'empty key does EXIST' );

# 11-13 = can we modify a value
$val = $hash{'test'};
$val->[1] = 4;
$hash{'test'} = $val;
tied(%hash)->flush();
$val = $hash{'test'};
&report_result( ref($val) eq "Goober" , "lost our bless on re-stored value" );
&report_result( $$val[2] == 3 , "array munged before store" );
&report_result( $$val[1] == 4 , "change to array didn't survive" );

# 14-15 = are loaded values unique
$hash{'one'} = [1, 2, 3];
$hash{'two'} = [4, 5, 6];
tied(%hash)->flush();
&report_result( $hash{one}->[0] == 1 , "array one has been overwritten" );
&report_result( $hash{two}->[0] == 4 , "array two has been overwritten" );

# 16-17 = check reload of external modified cached value
sleep 1; # make sure file timestamps are different
if (open(DATA, ">$dir/one")) {
   print DATA <<'END_DATA';
$data = [
  7,
  8,
  9
];

1;
END_DATA
   close(DATA);
   &report_result(1, "could not overwrite value stored on disk" );
} else {
   &report_result(0, "could not overwrite value stored on disk" );
}
&report_result( $hash{one}->[0] == 7 , 
                "cached value remains even though stored value is newer" );

# 18 = grind it for a while
print "Test 18 will take a little time... be patient\n" if ($ENV{TEST_VERBOSE});
for $i (1..100) {
   $hash{$i} = [ $i, 234 ];
}
for $i (1..100) {
   if ($hash{$i}->[0]) {
      $j = $i-1;
   }
   $hash{$i} = [ $j, 22 ];
}
$isok = 1;
for $i (1..100) {
   $isok = 0 if ($hash{$i}->[0] != ($i-1));
   $isok = 0 if ($hash{$i}->[1] != 22);
}
&report_result( $isok, "grinding caused damage" );

untie %hash;
`rm -rf data`;
